# desynch.tcl for netbots.tcl v3.15
# designed to work with eggdrop 1.5.3 or higher
# Johoho's Eggdrop Page - http://johoho.eggdrop.cx
# desynch.tcl copyright (c) 1999,2000 by johoho <johoho@hojo-net.de>
# netbots.tcl copyright (c) 1998-2000 by slennox <slennox@egghelp.org>

## desynch.tcl component script v1.10.1, 19.03.2000 ##

# History:
# 
# v1.00.0, 27.12.1999 - initial release
# v1.10.0, 27.02.2000 - fixed lot's of bugs
# v1.10.1, 19.03.2000 - changed header and readme :)

# Todo:
# 
# - make desynch check work
# - comparison of topics and modes so there'd be no not needed traffic

# thanks to daimonic for his help. thanks slennox for netbots.tcl

proc ds_helpidx {hand chan idx} {
  if {![matchattr $hand o|o $chan]} {return 0}
  putidx $idx "desynch.tcl commands"
  if {[matchattr $hand n]} {
    putidx $idx " For owners:"
    putidx $idx "  desynchcheck  desynchremove"
  }
  putidx $idx " "
  return 1
}

proc ex_help {hand chan idx cmd} {
  if {[matchattr $hand o|o $chan]} {
    switch -exact -- $cmd {
      "desynchcheck" {
        if {[matchattr $hand n]} {
          putidx $idx "# desynchcheck <#channel>"
          putwrap $idx 3 "This commands checks the specified channel for desynch. It does so by comparing user / op lists, topic, ban lists, exempts and invites."
          return 1
        }
      }
      "desynchremove" {
        if {[matchattr $hand n]} {
          putidx $idx "# desynchremove <#channel> \[crusty\]"
          putwrap $idx 3 "This command tries to remove desynch by synching topic and modes on channel. Optional option 'crusty' should be used only if it doesn't work without. It'll close the channel completely and cycle it.\n Please note, that you must enable all bots in #channel via netcontrol."
          return 1
        }
      }
    }
  }
  return 0
}

lappend nb_helpidx "ds_helpidx"
set nb_help(desynchcheck) "ds_help"
set nb_help(desynchremove) "ds_help"

proc ds_check {hand idx arg} {
global nb_control

}

proc ds_netcheck {frombot arg} {
  set $ds_nick [slindex $arg 0]
  if {[isbotnick $ds_nick]} {
    return 1
  } else {
    return 0
  }
}

proc ds_remove {hand idx arg} {
global nb_control ds_fcnt botnick
  set ds_fcnt 0
  set ds_crusty 0
  set ds_chan ""
  set ds_chan [lindex [split $arg] 0]
  set ds_tmp [lindex [split $arg] 1]
  if {$ds_tmp == "crusty"} {
    set ds_crusty 1
  }
  if {$ds_chan != ""} {
    if {[validchan $ds_chan]} {
      foreach i [chanlist $ds_chan] {
        incr ds_fcnt
      }
      putcmdlog "#$hand# desynchremove $ds_chan $ds_crusty"
      nb_sendcmd $nb_control ds_netremove "$hand $idx $ds_chan $ds_crusty"
    } else {
      putdcc $idx "$ds_chan is not a valid channel"
      return 0
    }
  } else {
    putdcc $idx "usage: .desynchremove <#channel> \[crusty\]"
    return 0
  }
}

proc ds_netremove2 {frombot arg} {
  set ds_crusty ds_fcnt
  set uidx [slindex $arg 1]
  set ds_chan [slindex $arg 2]
  set ds_crusty [slindex $arg 3]
  set ds_chancnt [slindex $arg 4]
  if {($ds_fcnt != $ds_chancnt)} {
    putdcc $uidx "it seems that there's a user desynch. better use crusty option."
  } else {
    if {($ds_crusty == "1")} {
      putdcc $uidx "Channel cleaned successfully with the crusty method."
    } else {
      putdcc $uidx "Modes and topic are synced again."
    }
  }
}

proc ds_netremove {frombot arg} {
global ds_topic
  set ds_crusty ""
  set hand [slindex $arg 0]
  set uidx [slindex $arg 1]
  set ds_chan [slindex $arg 2]
  set ds_crusty [slindex $arg 3]
  putlog "netbots \[$hand@$frombot\]: desynchremove $ds_chan $ds_crusty"
  if {$ds_crusty == "1"} {
    putlog "DEBUG: CRUSTY mode wanted"
  } else {
    # op all ops
    foreach i [chanlist $ds_chan] {
      if {[matchattr $i n|n] || [matchattr $i b]} {
        if {(![isop $i $ds_chan])} {
          pushmode $ds_chan +o $i
        }
      } elseif {[isop $i $ds_chan]} {
        pushmode $ds_chan -o $i
      }
    }
    # devoice all users
    foreach i [chanlist $ds_chan] {
      if {[isvoice $i $ds_chan]} {
        pushmode $ds_chan -v $i
      }
    }
    # remove all +b modes
    foreach i [chanbans $ds_chan] {
      set tmp [lindex [split $i] 0]
      if {[isbansticky $tmp $ds_chan]} {
        unstick $tmp
      }
      pushmode $ds_chan -b $tmp
    }
    # remove all +e modes
    foreach i [chanexempts $ds_chan] {
      set tmp [lindex [split $i] 0]
      if {[isexemptsticky $tmp $ds_chan]} {
        unstickexempt $tmp
      }
      pushmode $ds_chan -e $tmp
    }
    # remove all +I modes
    foreach i [chaninvites $ds_chan] {
      set tmp [lindex [split $i] 0]
      if {[isinvitesticky $tmp $ds_chan]} {
        unstickinvite $tmp
      }
      pushmode $ds_chan -I $tmp
    }
    # remove +l mode
    pushmode $ds_chan -l
    # set topic
    putserv "TOPIC $ds_chan :$ds_topic"
    # okay, seems that everything worked fine
    putlog "channel sync complete - ops, voices, bans, exempts, invites and topic are synced again. hopefully."
    set ds_chancnt 0
    foreach i [chanlist $ds_chan] {
      incr ds_chancnt
    }
#    nb_sendcmd $frombot ds_netremove2 "$hand $uidx $ds_chan $ds_crusty $ds_chancnt"
    timer 3 "reload"
  }
}



set nb_netcmds(ds_netremove) "ds_netremove"
set nb_netcmds(ds_netremove2) "ds_netremove2"
set nb_netcmds(ds_netseccheck) "ds_netcheck"

bind dcc n desynchcheck ds_check
bind dcc n desynchremove ds_remove
